;************************************************************** 
;* 
;* Pinbelegung 
;* ----------------------------------  
;* PORTA: 0 
;*        1  
;*        2  
;*        3  
;*        4  
;* PORTB: 0 FSK = frequency shift keying = Modulationseingang  
;*        1 nSEL = Whlt das RFM 02 aus  
;*        2 SCK = seriell Clock = Taktung fr die SDI Schnittstelle  
;*        3 SDI = seriell data in = Serieller Dateneingang  
;*        4   
;*        5   
;*        6  
;*        7 
;* PORTC: 0  nIRQ = interrupt Request = Interruptanfrage aus dem RFM 02
;*        1  
;*        2 
;*        3  
;*        4  
;*        5  
;*        6  
;*        7  
;* 
;************************************************************** 
; 
; 
; 
;************************************************************** 
 
; Includedatei fr den 16F876 einbinden

	list p=16f876
	#include <P16f876.INC>

	ERRORLEVEL      -302    	;SUPPRESS BANK SELECTION MESSAGES

; Configuration festlegen

; Power on Timer, kein Watchdog, XT-Oscillator, kein Brown out, kein LV-programming
	__CONFIG	_PWRTE_ON & _WDT_OFF & _HS_OSC & _BODEN_OFF & _LVP_OFF

;**************************************************************
 


; Variablen 
zaehler1		EQU     0x20 
zaehler2		EQU		0x21
zaehler3		EQU		0x22
daten			EQU		0x23
loopcount		EQU		0x24

; Constanten festlegen

Ini_con Equ	B'00000000'	; TMR0 -> Intetupt disable
Ini_opt	Equ	B'00000010'	; pull-up

#define	LED3	PORTB,7	; 
#define	LED2	PORTB,6	; 
#define	LED1	PORTB,5	; 
#define	LED0	PORTB,4	; 
#define	SDI	PORTB,3		; SDI = seriell data in = Serieller Dateneingang
#define	SCK	PORTB,2		; SCK = seriell Clock = Taktung fr die SDI Schnittstelle
#define	SEL	PORTB,1		; nSEL = Whlt das RFM 02 aus
#define	SDO	PORTC,1		; Data from Modul
#define	IRQ	PORTC,0		; nIRQ = interrupt Request = Interruptanfrage aus dem RFM 02


; Anfangsinitialisierung 

init 
; Ports vorbereiten 
        bsf     STATUS, RP0    ; auf Bank 1 umschalten

		movlw   Ini_opt     		; pull-up on
		movwf   OPTION_REG 
		movlw	B'11111111'			; PortC alle Inputs
		movwf	TRISC				
		movlw	B'00000000'			; PortB alle outputs 
		movwf	TRISB				
        movlw   B'00000000'    ; PortA alle output 
        movwf   TRISA 
        bcf     STATUS, RP0    ; auf Bank 0 zurckschalten 
		clrf	PORTA			
        clrf    PORTB			
	
		BCF     STATUS,RP0     ; Bank0 

		clrf	zaehler1 
		clrf	zaehler2

;********************************************************** 
; Hauptprogrammschleife 
		
Init 

		call 	warten2	
		call 	warten2	
		call 	warten2	
		call 	warten2	
		call 	warten2	
		call 	warten2	
		call 	warten2	
		call 	warten2	
		call 	warten2	
		call 	warten2	
		bsf		SEL						; Grundzustand der Selekt-Leitung
		call 	warten2	
		bcf		SCK						; Grundzustand der Clock -Leitung
		call	warten
		bcf		SEL						; Modul selektieren
		call 	warten	


; *******************************************************************************
 ;  C2E0  CLK: 10MHz    
		movlw	0xC2
		movwf	daten
		call	WriteCMD
		movlw	0xE0
		movwf	daten
		call	WriteCMD
; **********************************************************************
;  C42B Data Filter: internal  
		movlw	0xC4
		movwf	daten
		call	WriteCMD
		movlw	0x2B
		movwf	daten
		call	WriteCMD
; ***************************************************************************
;  CE88  FIFO mode
		movlw	0xCE
		movwf	daten
		call	WriteCMD
		movlw	0x88
		movwf	daten
		call	WriteCMD
; ********************************************************************************
 ;  C6F7 AFC settings: autotuning: -10kHz...+7,5kHz  --------------------
		movlw	0xC6
		movwf	daten
		call	WriteCMD
		movlw	0xF7
		movwf	daten
		call	WriteCMD
; **************************************************************************************
 ;  E000 disable wakeuptimer  -------------------------------
		movlw	0xE0
		movwf	daten
		call	WriteCMD
		movlw	0x00
		movwf	daten
		call	WriteCMD
; **************************************************************************************************
;  CC00 disable low duty cycle ------------------------------
		movlw	0xCC
		movwf	daten
		call	WriteCMD
		movlw	0x00
		movwf	daten
		call	WriteCMD
; **************************************************************************************************
 ;  A620   set freqenz  ---------------
		movlw	0xA6
		movwf	daten
		call	WriteCMD
		movlw	0x20
		movwf	daten
		call	WriteCMD
; ********************************************************************************
 ; 8978 Bandbreite -----------
		movlw	0x89
		movwf	daten
		call	WriteCMD
		movlw	0x78
		movwf	daten
		call	WriteCMD
; ********************************************************************************
 ;  C811  baudrate    --gendert auf C823 -------------------------------
		movlw	0xC8
		movwf	daten
		call	WriteCMD
		movlw	0x23
		movwf	daten
		call	WriteCMD
; **********************************************************************************************
;  C0E9 RX on ------------------------------
		movlw	0xC0
		movwf	daten
		call	WriteCMD
		movlw	0xE9
		movwf	daten
		call	WriteCMD
; ************************************************************************************************
; CE89 FIFO
		movlw	0xCE
		movwf	daten
		call	WriteCMD
		movlw	0x89
		movwf	daten
		call	WriteCMD
;  CE8B enable FIFO
		movlw	0xCE
		movwf	daten
		call	WriteCMD
		movlw	0x8B
		movwf	daten
		call	WriteCMD
;*****************************************************************************
Main
			call 	warten2	
			bsf SEL
			Call	warten2
			bcf	SEL
			call	warten2
			bcf SDI
WaitSD0
		btfss	SDO
		goto WaitSD0
Readloop
		movlw	D'16'
		movwf	loopcount				;Schleife 16 mal durchlaufen
startRead
		bsf		SCK
		call warten
		bcf		SCK
		call warten
		decf	loopcount
		btfss	STATUS,Z
		goto	startRead
		call warten

LeseData 
		movlw	D'8'
		movwf	loopcount				;Schleife 8 mal durchlaufen
		clrf	daten
NextBit
		bsf		SCK
		btfsc	SDO
		bsf		daten,0
		call warten
		bcf		SCK
		call warten
		rlf		daten,1
		decf	loopcount
		btfss	STATUS,Z
		goto NextBit
		call warten
		call warten

		btfss	daten,0
		bcf		LED0
		btfss	daten,1
		bcf		LED1
		btfss	daten,2
		bcf		LED2
		btfss	daten,3
		bcf		LED3
		btfsc	daten,0
		bsf		LED0
		btfsc	daten,1
		bsf		LED1
		btfsc	daten,2
		bsf		LED2
		btfsc	daten,3
		bsf		LED3


		movlw	D'8'
		movwf	loopcount				;Schleife 8 mal durchlaufen
		clrf	daten
NextBit2
		bsf		SCK
		btfsc	SDO
		bsf		daten,0
		call warten
		bcf		SCK
		call warten
		rlf		daten,1
		decf	loopcount
		btfss	STATUS,Z
		goto NextBit2
		call warten
		call warten

		btfss	daten,0
		bcf		LED0
		btfss	daten,1
		bcf		LED1
		btfss	daten,2
		bcf		LED2
		btfss	daten,3
		bcf		LED3
		btfsc	daten,0
		bsf		LED0
		btfsc	daten,1
		bsf		LED1
		btfsc	daten,2
		bsf		LED2
		btfsc	daten,3
		bsf		LED3

; 0xC0E8  Empfnger aus 
		bsf		SEL
		call warten2
		bcf		SEL

		movlw	0xC0
		movwf	daten
		call	WriteCMD
		movlw	0xE8
		movwf	daten
		call	WriteCMD

		goto Init
;----------------------------------------------------------------------------------------
WriteCMD
		movlw	D'8'
		movwf	loopcount				;Schleife 8 mal durchlaufen
startWrite
		rlf		daten, 1				;Daten nach links schieben, Bit 8 geht dann in den berlauf
		btfss	STATUS,C				; springe wenn C=1
		goto	send_0					; bertrage eine 0
		bsf		SDI
		goto 	endWrite
send_0	bcf		SDI			
endWrite 
		call 	Takt
		decf	loopcount
		btfss	STATUS,Z
		goto	startWrite
		return
;------------------------------------------------------------------------------------------

WaitInt btfss	IRQ
		goto	WaitInt
waitLow		btfsc	IRQ
		goto 	waitLow
		return


		
Takt
		call	warten
		bsf		SCK
		call	warten
		bcf		SCK
		call	warten
		return

warten
		movlw	D'1'					;Work mit Dezimal 1 laden
		movwf	zaehler3				; Zhlerstand setzen
		movlw	D'20'					;Work mit Dezimal 20 laden
		movwf	zaehler2				; Zhlerstand setzen		


schleife_intern
		;DECFSZ	zaehler1, f
		;goto		schleife_intern 
		DECFSZ	zaehler2, f 
		goto		schleife_intern
		DECFSZ	zaehler3, f 
		goto		schleife_intern
		return 

warten2
		movlw	D'5'					;Work mit Dezimal 5 laden
		movwf	zaehler3				; Zhlerstand setzen
schleife_intern2
		;DECFSZ	zaehler1, f
		;goto		schleife_intern2
		DECFSZ	zaehler2, f 
		goto		schleife_intern2
		DECFSZ	zaehler3, f 
		goto		schleife_intern2
		return 

	end


;**********************************************************
