;************************************************************** 
;* 
;* Pinbelegung 
;* ----------------------------------  
;* PORTA: 0 
;*        1  
;*        2  
;*        3  
;*        4  
;* PORTB: 0 FSK = frequency shift keying = Modulationseingang  
;*        1 nSEL = Whlt das RFM 02 aus  
;*        2 SCK = seriell Clock = Taktung fr die SDI Schnittstelle  
;*        3 SDI = seriell data in = Serieller Dateneingang  
;*        4   
;*        5   
;*        6  
;*        7 
;* PORTC: 0  nIRQ = interrupt Request = Interruptanfrage aus dem RFM 02
;*        1  
;*        2 
;*        3  
;*        4  
;*        5  
;*        6  
;*        7  
;* 
;************************************************************** 
; 
; 
; 
;************************************************************** 
 
; Includedatei fr den 16F876 einbinden

	list p=16f876
	#include <P16f876.INC>

	ERRORLEVEL      -302    	;SUPPRESS BANK SELECTION MESSAGES

; Configuration festlegen

; Power on Timer, kein Watchdog, XT-Oscillator, kein Brown out, kein LV-programming
	__CONFIG	_PWRTE_ON & _WDT_OFF & _HS_OSC & _BODEN_OFF & _LVP_OFF

;**************************************************************
 


; Variablen 
zaehler1		EQU     0x20 
zaehler2		EQU		0x21
zaehler3		EQU		0x22
daten			EQU		0x23
loopcount		EQU		0x24

; Constanten festlegen

Ini_con Equ	B'00000000'	; TMR0 -> Intetupt disable
Ini_opt	Equ	B'00000010'	; pull-up


#define	SDI	PORTB,3		; SDI = seriell data in = Serieller Dateneingang
#define	SCK	PORTB,2		; SCK = seriell Clock = Taktung fr die SDI Schnittstelle
#define	SEL	PORTB,1		; nSEL = Whlt das RFM 02 aus
#define	FSK	PORTB,0		; FSK = frequency shift keying = Modulationseingang
#define	IRQ	PORTC,0		; nIRQ = interrupt Request = Interruptanfrage aus dem RFM 02
#define	S0	PORTC,4		; 
#define	S1	PORTC,5		; 
#define	S2	PORTC,6		; 

; Anfangsinitialisierung 

init 
; Ports vorbereiten 
        bsf     STATUS, RP0    ; auf Bank 1 umschalten

		movlw   Ini_opt     		; pull-up on
		movwf   OPTION_REG 
		movlw	B'11111111'			; PortC alle Inputs
		movwf	TRISC				
		movlw	B'00000000'			; PortB alle outputs 
		movwf	TRISB				
        movlw   B'00000000'    ; PortA alle output 
        movwf   TRISA 
        bcf     STATUS, RP0    ; auf Bank 0 zurckschalten 
		clrf	PORTA			
        clrf    PORTB			
	
		BCF     STATUS,RP0     ; Bank0 

		clrf	zaehler1 
		clrf	zaehler2

;********************************************************** 
; Hauptprogrammschleife 
	
Main 
	call 	warten2	
	call 	warten2	
	call 	warten2	
	call 	warten2	
	call 	warten2	
	call 	warten2	
	call 	warten2	
	call 	warten2	
	call 	warten2	
	call 	warten2	
	call 	warten2	
	 
		bsf		SEL						; Grundzustand der Selekt-Leitung
		bcf		SCK						; Grundzustand der Clock -Leitung
		call	warten2
		bcf		SEL						; Modul selektieren
		call 	warten2	
;  ????
;  C0E0 
		movlw	0xC0
		movwf	daten
		call	WriteCMD
		movlw	0xE0
		movwf	daten
		call	WriteCMD

;   Config. Setting commands (CLK - Pin - Frequenz, ...)
;   8F80 
		movlw	0x8F
		movwf	daten
		call	WriteCMD
		movlw	0x80
		movwf	daten
		call	WriteCMD
;   ????
;   C2A0
		movlw	0xC2
		movwf	daten
		call	WriteCMD
		movlw	0xA0
		movwf	daten
		call	WriteCMD		

;   Semdefrequenz
;   A620
		movlw	0xA6
		movwf	daten
		call	WriteCMD
		movlw	0x20
		movwf	daten
		call	WriteCMD

;   ????
;  B400
		movlw	0xB4
		movwf	daten
		call	WriteCMD
		movlw	0x00
		movwf	daten
		call	WriteCMD	
;   8F80 
		movlw	0x8F
		movwf	daten
		call	WriteCMD
		movlw	0x80
		movwf	daten
		call	WriteCMD	

;   D2C0 
		movlw	0xD2
		movwf	daten
		call	WriteCMD
		movlw	0xC0
		movwf	daten
		call	WriteCMD

;   Baudrate
;   C811    gendert auf C823
		movlw	0xC8
		movwf	daten
		call	WriteCMD
		movlw	0x23
		movwf	daten
		call	WriteCMD


;   Power - Amp., Disable CLK-Pin, ...
;   C038 
		movlw	0xC0
		movwf	daten
		call	WriteCMD
		movlw	0x38
		movwf	daten
		call	WriteCMD
;----------------------------------------------------------------------------------------------------------
		bsf SEL
		call warten2
		bcf SEL

		movlw 0xC6
		movwf	daten
		call 	WriteCMD 

;----------------------------------------------------------------------------------------------------------



; preamble
		movlw	0xAA
		movwf	daten
		call 	WriteDATA
		movlw	0xAA
		movwf	daten
		call 	WriteDATA
		movlw	0xAA
		movwf	daten
		call 	WriteDATA

; Frame (0x2DD4)bertragen
		movlw	0x2D
		movwf	daten
		call 	WriteDATA
		movlw	0xD4
		movwf	daten
		call 	WriteDATA


; Daten senden

WriteDaten
		clrf	daten
		btfsc	S0
		bsf		daten,0
		btfsc	S1
		bsf		daten,1
		btfsc	S2
		bsf		daten,2
		call 	WriteDATA


		clrf	daten
		btfsc	S0
		bsf		daten,0
		btfsc	S1
		bsf		daten,1
		btfsc	S2
		bsf		daten,2
		call 	WriteDATA


;   0xC464    schaltet den Sender wieder aus 

		call warten2
		bsf		SEL
		call warten2
		bcf		SEL
		
		movlw	0xC4
		movwf	daten
		call	WriteCMD
		movlw	0x64
		movwf	daten
		call	WriteCMD


		goto 	Main


;-----------------------------------------------------------------------------------------------------------
WriteDATA
		movlw	D'8'
		movwf	loopcount				;Schleife 8 mal durchlaufen
startWriteD
		rlf		daten, 1				;Daten nach links schieben, Bit 8 geht dann in den berlauf
		btfss	STATUS,C				; springe wenn C=1
		goto	sendD_0					; bertrage eine 0

		bsf		SDI
		goto 	WaitInt
sendD_0	
		bcf		SDI	

WaitInt btfss	IRQ
		goto	WaitInt
waitLow	btfsc	IRQ
		goto 	waitLow

		decf	loopcount
		btfss	STATUS,Z
		goto	startWriteD
		return

;-----------------------------------------------------------------------------------------------------------

WriteCMD
		movlw	D'8'
		movwf	loopcount				;Schleife 8 mal durchlaufen
startWrite
		rlf		daten, 1				;Daten nach links schieben, Bit 8 geht dann in den berlauf
		btfss	STATUS,C				; springe wenn C=1
		goto	send_0					; bertrage eine 0
		bsf		SDI
		goto 	endWrite
send_0	bcf		SDI			
endWrite 
		call 	Takt
		decf	loopcount
		btfss	STATUS,Z
		goto	startWrite
		return

;-----------------------------------------------------------------------------------------------------------

Takt
		call	warten
		bsf		SCK
		call	warten
		bcf		SCK
		call	warten
		return
;-----------------------------------------------------------------------------------------------------------



warten
		movlw	D'1'					;Work mit Dezimal 1 laden
		movwf	zaehler3				; Zhlerstand setzen
		movlw	D'20'					;Work mit Dezimal 20 laden
		movwf	zaehler2				; Zhlerstand setzen		


schleife_intern
		;DECFSZ	zaehler1, f
		;goto		schleife_intern 
		DECFSZ	zaehler2, f 
		goto		schleife_intern
		DECFSZ	zaehler3, f 
		goto		schleife_intern
		return 

warten2
		movlw	D'5'					;Work mit Dezimal 5 laden
		movwf	zaehler3				; Zhlerstand setzen
schleife_intern2
		;DECFSZ	zaehler1, f
		;goto		schleife_intern2
		DECFSZ	zaehler2, f 
		goto		schleife_intern2
		DECFSZ	zaehler3, f 
		goto		schleife_intern2
		return 

	end


;**********************************************************
